/**
	@file qw_util.h
	@brief header for qw util 
**/

#ifndef __QW_UTIL_H__
#define __QW_UTIL_H__

#include "hx_types.h"

HX_UINT32 get_uptime(void);

void QWUtil_version(void);

unsigned long get_time_in_milisecond(void);

const char * get_time_string(void);
const char * get_time_string_ex(void);
int int_system (char *command);

#define QW_TIMEMSG(fmt, args...) \
	fprintf(stderr, "[QWTIME_DEBUG:%s] %s:%d ## " fmt, get_time_string(), __func__, __LINE__, ##args); fflush(stderr)

#endif //__QW_UTIL_H__

