#ifndef _S_LIST_H
#define _S_LIST_H

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

/*
 * Simple doubly linked list implementation.
 *
 * Some of the internal functions ("__xxx") are useful when
 * manipulating whole lists rather than single entries, as
 * sometimes we already know the next/prev entries and we can
 * generate better code by using them directly rather than
 * using the generic single-entry routines.
 */

struct list_head {
	struct list_head *next, *prev;
};

#define LIST_HEAD_INIT(name)	{ &(name), &(name) }

#define LIST_HEAD(name) \
	struct list_head name = LIST_HEAD_INIT(name)

#define INIT_LIST_HEAD(ptr) \
	do { \
		(ptr)->next = (ptr); (ptr)->prev = (ptr); \
	} while (0)

/*
 * Insert a new head between two known consecutive entries. 
 *
 * This is only for internal list manipulation where we know
 * the prev/next entries already!
 */
 __inline__ void 
__list_move(struct list_head * newone, struct list_head * prev,
	struct list_head * next);

/**
 * list_move - move list entris to new list
 * @head: the new list
 */
 __inline__ void list_move(struct list_head *old, 
	struct list_head *newone);

/*
 * Insert a new entry between two known consecutive entries. 
 *
 * This is only for internal list manipulation where we know
 * the prev/next entries already!
 */
 __inline__ void __list_add(struct list_head * newone,
	struct list_head * prev, struct list_head * next);

/**
 * list_add_prev - add a new entry in previous entry
 * @new: new entry to be added
 * @head: list head to add it after
 */
 __inline__ void list_add_prev(struct list_head *newone, 
	struct list_head *head);

/**
 * list_add - add a new entry
 * @new: new entry to be added
 * @head: list head to add it after
 *
 * Insert a new entry after the specified head.
 * This is good for implementing stacks.
 */
 __inline__ void list_add(struct list_head *newone, 
	struct list_head *head);

/**
 * list_add_tail - add a new entry
 * @new: new entry to be added
 * @head: list head to add it before
 *
 * Insert a new entry before the specified head.
 * This is useful for implementing queues.
 */
 __inline__ void list_add_tail(struct list_head *newone, 
	struct list_head *head);

/*
 * Delete a list entry by making the prev/next entries
 * point to each other.
 *
 * This is only for internal list manipulation where we know
 * the prev/next entries already!
 */
 __inline__ void __list_del(struct list_head *prev,
	struct list_head *next);

/**
 * list_del - deletes entry from list.
 * @entry: the element to delete from the list.
 * Note: list_empty on entry does not return true after this, the entry is in an undefined state.
 */
 __inline__ void list_del(struct list_head *entry);

/**
 * list_del_init - deletes entry from list and reinitialize it.
 * @entry: the element to delete from the list.
 */
 __inline__ void list_del_init(struct list_head *entry);

/**
 * list_empty - tests whether a list is empty
 * @head: the list to test.
 */
 __inline__ int list_empty(struct list_head *head);

/**
 * list_entry - get the struct for this entry
 * @ptr:	the &struct list_head pointer.
 * @type:	the type of the struct this is embedded in.
 * @member:	the name of the list_struct within the struct.
 */
#define list_entry(ptr, type, member) \
	((type *)((char *)(ptr)-(unsigned long)(&((type *)0)->member)))

/**
 * list_for_each	-	iterate over a list
 * @pos:	the &struct list_head to use as a loop counter.
 * @head:	the head for your list.
 */
#define list_for_each(pos, head) \
	for (pos = (head)->next; pos != (head); pos = pos->next)
 
/**
 * list_for_each_prev	-	iterate over a list backwards
 * @pos:	the @struct list head to use as a loop counter.
 * @head:	the head for your list.
 */
#define list_for_each_prev(pos, head) \
	for (pos = (head)->prev; pos != head; pos = pos->prev)

/**
 * list_for_each_safe	-	iterate over a list safe against removal of list entry
 * @pos:	the &struct list_head to use as a loop counter.
 * @n:		another &struct list_head to use as temporary storage
 * @head:	the head for your list.
 */
#define list_for_each_safe(pos, n, head) \
	for (pos = (head)->next, n = pos->next; pos != (head); \
		pos = n, n = pos->next)

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // S_LIST_H
