#ifndef _CHANNELSDK_H
#define _CHANNELSDK_H

#include "DTV.h"
#include "AZBoxTVSDK_DEF.h"


int ChannelSDK_Init(void);
int ChannelSDK_Load(const char *fname);
int ChannelSDK_Save(const char *fname);
struct channel_st * ChannelSDK_CreateChannel(void);
int ChannelSDK_Destroy(void);
struct channel_list_st *ChannelSDK_GetChannelList(void);
void ChannelSDK_Version(void);

int ChannelSDK_InitNetwork(void);
HX_STATUS ChannelSDK_LoadNetwork (HX_UINT type, const char *fname);
HX_STATUS ChannelSDK_SaveNetwork (HX_UINT type, const char *fname);
struct all_networks_st *ChannelSDK_GetNetworkList(void);

#endif
